/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2009 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 *
 *******************************************************************************
 *
 * $Id: PropertyPersistenceBeanAccessor.java,v b89b0b568aeb 2010/02/17 15:26:52 fabrizio $
 *
 ******************************************************************************/
package it.tidalwave.metadata.persistence.spi;

import it.tidalwave.util.logging.Logger;
import javax.annotation.Nonnull;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: PropertyPersistenceBeanAccessor.java,v b89b0b568aeb 2010/02/17 15:26:52 fabrizio $
 *
 ******************************************************************************/
public class PropertyPersistenceBeanAccessor<Bean> extends MistralBeanAccessor<Bean>
  {
    private static final String CLASS = PropertyPersistenceBeanAccessor.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);
            
    @Nonnull
    private final String resourceName;
    
    private final Map<String, String> codeMapByPropertyName = new HashMap<String, String>();
    
    private final Map<String, String> propertyNameMapByCode = new HashMap<String, String>();
    
    /***************************************************************************
     *
     * 
     * 
     **************************************************************************/
    public PropertyPersistenceBeanAccessor (@Nonnull final Class<Bean> beanClass, 
                                            @Nonnull final String propertySetName, 
                                            @Nonnull final InputStream is) 
      throws IOException
      {
        super(beanClass, propertySetName);
        this.resourceName = "";
        loadMapping(is);
      }
    
    /***************************************************************************
     *
     * 
     * 
     **************************************************************************/
    public PropertyPersistenceBeanAccessor (@Nonnull final Class<Bean> beanClass,
                                            @Nonnull final String propertySetName, 
                                            @Nonnull final Class<?> clazz,
                                            @Nonnull final String resourceName)
      throws IOException
      {
        super(beanClass, propertySetName);
        this.resourceName = resourceName;
        InputStream is = null;
        
        try
          {
            is = clazz.getResourceAsStream(resourceName);
            
            if (is == null)
              {
                throw new FileNotFoundException("Resource not found: " + resourceName);
              }
            
            loadMapping(is);
          }
        catch (IOException e)
          {
            throw e;
          }
        finally
          {
            try
              {
                if (is != null)
                  {
                    is.close();
                  }
              }
            catch (Exception e)
              {
                logger.warning("Exception in close(): " + e);
              }  
          }
      }
            
    /***************************************************************************
     *
     * 
     * 
     **************************************************************************/
    @Nonnull 
    public String findCode (@Nonnull final String propertyName)
      {
        final String code = codeMapByPropertyName.get(propertyName);
        
        if (code == null)
          {
            // FIXME: shouldn't be UnknownPropertyException?
            throw new IllegalArgumentException(String.format("Not found: %s - available: %s", propertyName, codeMapByPropertyName));
          }
        
        return code;
      }
    
    /***************************************************************************
     *
     * 
     * 
     **************************************************************************/
    @Nonnull 
    public String findPropertyName (@Nonnull final String code)
      {
        final String propertyName = propertyNameMapByCode.get(code);
        
        if (propertyName == null)
          {
            // FIXME: shouldn't be UnknownPropertyException?
            throw new IllegalArgumentException(String.format("Not found: %s - available: %s", code, propertyNameMapByCode));
          }
        
        return propertyName;
      }

    /***************************************************************************
     *
     * 
     * 
     **************************************************************************/
    private void loadMapping (@Nonnull final InputStream is) 
      throws IOException 
      {
        final Properties properties = new Properties();
        properties.load(is);
        
        for (final Enumeration<String> e = (Enumeration<String>)properties.propertyNames(); e.hasMoreElements(); )
          {
            final String propertyName = e.nextElement().trim();
            final String code = properties.getProperty(propertyName).trim();
            codeMapByPropertyName.put(propertyName, code);
            propertyNameMapByCode.put(code, propertyName);
          }
      }
    
    /***************************************************************************
     *
     * 
     * 
     **************************************************************************/
    @Override
    @Nonnull 
    public String toString()
      {
        return String.format("PropertyPersistenceCodeProvider[%s, %s]", getPropertySetName(), resourceName);    
      }
  }
